package com.sitemesh.taglib;

import com.sitemesh.Page;
import com.sitemesh.RequestConstants;
import javax.servlet.ServletException;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * Convenience implementation of Tag containing generice methods required
 * by all (or most) taglibs.
 *
 * @author <a href="mailto:joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.5 $
 */
public abstract class AbstractTag implements Tag, RequestConstants {

	protected PageContext pageContext;
	protected Tag parent;

	/**
	 * To be implemented by all empty tags.
	 */
	public abstract int doEndTag() throws JspException;

	/**
	 * Returns SKIP_BODY.
	 */
	public int doStartTag() {
		return SKIP_BODY;
	}

	public void release() {
	}

	public Tag getParent() {
		return parent;
	}

	public void setParent( Tag parent ) {
		this.parent = parent;
	}

	public void setPageContext( PageContext pageContext ) {
		this.pageContext = pageContext;
	}

	/**
	 * Return the Page object from the PAGE scope. If this is found in REQUEST scope
	 * instead, it will be moved into PAGE scope - to handle multi-level includes.
	 */
	protected Page getPage() {
		Page p = ( Page )pageContext.getAttribute( PAGE, PageContext.PAGE_SCOPE );
		if ( p == null ) {
			p = ( Page )pageContext.getAttribute( PAGE, PageContext.REQUEST_SCOPE );
			pageContext.setAttribute( PAGE, p, PageContext.PAGE_SCOPE );
			pageContext.setAttribute( PAGE, null, PageContext.REQUEST_SCOPE );
		}
		return p;
	}

	/**
	 * Log exception generated by taglib.
	 */
	protected void trace( Exception e ) {
		e.printStackTrace();
	}

}
